from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library
standard_library.install_aliases()

DEFAULT_ORDER = ['renderers', 'operators', 'initializers', 'emitters',
                 'children', 'forces', 'constraints',
                 'attributes', 'particleSystemDefinitions']
OPERATORS = ['renderers', 'operators', 'initializers', 'emitters',
             'children', 'forces', 'constraints']



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from builtins import open
from future import standard_library
standard_library.install_aliases()

from valvepcf.classes import *
from valvepcf.loader import load_pcf
from valvepcf.unloader import unload_pcf, save_pcf


class Pcf(PcfRootNode):
    """
    This is the basic class to interact with pcf files.
    To interact with the data, consult :py:class:`PcfRootNode<PcfRootNode>`
    which this class inherits
    """

    def __init__(self, path=None):
        """
        initalize a pcf file.

        :param path: The location of the pcf file to be parsed
        :type path: str
        """
        self.source_path = path  #: :type: (str) - The location of the parsed file.
        self._data = None  # raw parsed data

        self.binary_format = None
        self.binary_version = None
        self.pcf_format = None
        self.pcf_version = None

        if self.source_path:
            load_pcf(self)

    def save(self, destination=None):
        """Saves the current instance of the Pcf. Overwrites original pcf file if no destination is provided.

        :param destination: A path (directory + filename) to determine where to save the pcf file.
        :type destination: str, optional
        """

        dest = destination or self.source_path

        save_pcf(self, dest)



from __future__ import division
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

from builtins import range
from builtins import int
from future import standard_library
standard_library.install_aliases()

from construct import *
import uuid

attribute_types = {
    1: Default(Int32sl, 0) * "ATTRIBUTE_ELEMENT",
    2: Default(Int32sl, 0) * "ATTRIBUTE_INTEGER",
    3: Default(Float32l, 0.0) * "ATTRIBUTE_FLOAT",
    4: Default(Flag, False) * "ATTRIBUTE_BOOLEAN",
    5: Default(Switch(lambda ctx: ctx._root.versions[0], {4: Int16ul, 5: Int32ul},
                      default=CString('ascii')), None) * "ATTRIBUTE_STRING",
    6: PrefixedArray(Int32ul, Byte) * "ATTRIBUTE_BINARY",
    7: Default(Float32l, 0.0) * "ATTRIBUTE_TIME",
    8: Default(Int8ul[4], [0, 0, 0, 255]) * "ATTRIBUTE_COLOR",
    9: Default(Float32l[2], [0.0, 0.0]) * "ATTRIBUTE_VECTOR2",
    10: Default(Float32l[3], [0.0, 0.0, 0.0]) * "ATTRIBUTE_VECTOR3",
    11: Default(Float32l[4], [0.0, 0.0, 0.0, 0.0]) * "ATTRIBUTE_VECTOR4",
    12: Default(Float32l[3], [0.0, 0.0, 0.0]) * "ATTRIBUTE_QANGLE",
    13: Default(Float32l[4], [0.0, 0.0, 0.0, 1.0]) * "ATTRIBUTE_QUATERNION",
    14: Float32l[4][4] * "ATTRIBUTE_MATRIX"
}

for n in range(1, 15):
    attribute_types[n + 14] = PrefixedArray(Int32ul, attribute_types[n])


def build_versions(versions):
    return "<!-- dmx encoding binary {} format {} {} -->\n".format(*versions)


def parse_versions(vstring):
    return (int(vstring[25]), vstring[34:37], int(vstring[38]))


CDmxAttribute = Struct(
    'attributeName' / Switch(this._root.versions[0], {5: Int32ul},
                             default=Int16ul) * 'String dictionary index',
    'attributeType' / Int8ul * "Determines the type of attributeData",
    'attributeData' / Switch(this.attributeType, attribute_types))

CDmxElement = Struct(
    'elementType' / Int16ul,
    'elementDesc' / If(lambda ctx: ctx._root.versions[0] == 5, Int16ul),
    'elementName' / Switch(lambda ctx: ctx._root.versions[0],
                           {4: Int16ul, 5: Int32ul}, default=CString('ascii')),
    'elementUUID' / Default(Bytes(16) * "Globally unique identifier",
                            lambda ctx: uuid.uuid4().bytes))

PCF = Struct(
    'versionString' / Rebuild(CString('ascii'),
                              lambda ctx: build_versions(ctx._root.versions)),
    'versions' / Computed(lambda ctx: parse_versions(ctx.versionString)),
    'strings' / PrefixedArray(Switch(this._root.versions[0],
                                     {4: Int32ul, 5: Int32ul},
                                     default=Int16ul), CString('ascii')),
    'elements' / PrefixedArray(Int32ul, CDmxElement),
    'attributes' / PrefixedArray(Int32ul, CDmxAttribute)[len_(this.elements)])



from setuptools import setup, find_packages
import re

with open('README.md') as f:
    re_omit = r'<!--- start pypi omit -->.*<!--- end pypi omit -->'
    long_description = re.sub(re_omit, '', f.read(), flags=re.S)

VERSION = 'v1.1.2'
if 'VERSION_PLACEHOLDER' in VERSION:
    VERSION = '0.0.0'

setup(
    name='valvepcf',
    packages=['valvepcf'],
    version=VERSION,
    description='A library to parse .PCF files (particle effect files for the Source engine).',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='gpl-3.0',
    author='Maxime Dupuis',
    author_email='mdupuis@hotmail.ca',
    url='https://pysourcesdk.github.io/ValvePCF/',
    project_urls={
        'Documentation': 'https://pysourcesdk.github.io/ValvePCF/',
        'Github': 'https://github.com/pySourceSDK/ValvePCF',
    },
    keywords=['pcf', 'source', 'sourcesdk', 'hammer', 'valve'],
    install_requires=['construct', 'future'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Operating System :: Unix',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows'
    ],
)
